use tokio::task;

use crate::args::Args;
use crate::data::{BotType, Coordinates, GameRules};
use crate::human_player_ws::ServerMessage;
use crate::server::start_server;
use crate::test::bot_client::ClientEndResult;
use crate::test::network_utils::wait_for_port;
use crate::test::{bot_client, TestPort};

fn check_strikes_are_linear(msg: &ServerMessage) {
    if let ServerMessage::RequestFire { status } = msg {
        let mut in_fire_location = true;
        for y in 0..status.rules.map_height {
            for x in 0..status.rules.map_width {
                let c = Coordinates::new(x as i32, y as i32);

                if in_fire_location {
                    in_fire_location = status.your_map.did_fire_at_location(c);
                } else if status.your_map.did_fire_at_location(c) {
                    println!("Your map:");
                    status.print_your_map();
                    println!("Opponent map:");
                    status.print_opponent_map();
                    panic!("Found invalid fire location for linear bot!");
                }
            }
        }
    }
}

#[tokio::test]
async fn full_game() {
    let _ = env_logger::builder().is_test(true).try_init();

    let local_set = task::LocalSet::new();
    local_set
        .run_until(async move {
            task::spawn_local(start_server(Args::for_test(TestPort::LinearBotFullGame)));
            wait_for_port(TestPort::LinearBotFullGame.port()).await;

            let res = bot_client::BotClient::new(TestPort::LinearBotFullGame.as_url())
                .with_rules(GameRules::random_players_rules().with_bot_type(BotType::Linear))
                .with_server_msg_callback(check_strikes_are_linear)
                .run_client()
                .await
                .unwrap();
            assert_eq!(res, ClientEndResult::Finished);
        })
        .await;
}
